<?php
/**
* UsageDB class
* Provides database functions for usage.php
* @author Nick Korbel <lqqkout13@users.sourceforge.net>
* @version 11-27-03
* @package DBEngine
*/
/**
* Base directory of application
*/
@define('BASE_DIR', dirname(__FILE__) . '/../..');
/**
* DBEngine class
*/
require_once(BASE_DIR . '/lib/DBEngine.class.php');

class UsageDB extends DBEngine {

	/**
	* Get the min and max dates of all reservations in the database
	* @param none
	* @return array of min and max dates
	*/
	function get_min_max() {
		// Get min and max dates of reservations in database
		$query = 'SELECT MIN(date) as min, MAX(date) as max
							FROM ' . $this->get_table('reservations');
		$result = $this->db->getRow($query);
	
		// Check query
		$this->check_for_error($result);
		
		
		if (count($result) <= 0) {		// If there are no reservations, set all dates to today
			list(
				$return['min']['mon'], $return['min']['day'], $return['min']['year'],
				$return['max']['mon'], $return['max']['day'], $return['max']['year']
				) 
				= split('-', date('m-d-Y-m-d-Y'));
		}
		else {							// Else, set min and max values
			list($return['min']['mon'], $return['min']['day'], $return['min']['year']) 
					= split('-', date('m-d-Y', $result['min']));
			list($return['max']['mon'], $return['max']['day'], $return['max']['year'])
					= split('-', date('m-d-Y', $result['max']));
			
			
		}
		
		return $return;
	}

	/**
	* Finds all reservations matching the given criteria
	* @param array $memberids memberids
	* @param array $machid machids
	* @param int startDate start date timestamp
	* @param int endDate end date timestamp
	* @param float $startTime start time
	* @param float $endTime end time
	* @return mixed array of reservations or false if none are found
	*/
	function get_reservations($memberids, $machids, $startDate, $endDate, $startTime, $endTime) {
		
		// Limit columns as much as possible to speed up query
		$query = 'SELECT l.memberid, l.fname, l.lname,
				rs.machid, rs.name,
				res.*
				FROM ' . $this->get_table('login') . ' as l, '
					. $this->get_table('resources') . ' as rs, '
					. $this->get_table('reservations') . ' as res';

		// Begin setting up WHERE clause of query using passed in dates/times
		$where = ' WHERE (res.date>=?)
					AND (res.date<=?)
					AND (res.startTime>=?)
					AND (res.endTime<=?) ';
		// Begin setting up values array for query
		$values = array($startDate, $endDate, $startTime, $endTime);
	
		// Construct ORDER clause
		$order = ' ORDER BY l.lname, l.fname,
					rs.name,
					res.date, res.startTime, res.endTime ';
		/**********************************************
		* Determine what users to search for
		* 
		* If the first item in the array is string "all",
		* then search on all users.
		* Else get each memberid passed in and search just
		* for those members.
		**********************************************/
		// Add "AND" to where clause
		$where .= ' AND ';
	
		if ($memberids[0] != 'all') {
			// Join on specific memebers
			$where .= '(l.memberid=?';
			// Push this value onto values array
			array_push($values, $memberids[0]);
		}
	
		if ( (count($memberids)>1) && ($memberids[0] != 'all') ) {
			for ($i=1; $i<count($memberids); $i++) {
				$where .= ' OR l.memberid=?';
				// Push this value onto values array
				array_push($values, $memberids[$i]);
			}
		}
		// Add "AND" if WHERE clause is not empty
		$where .= ($memberids[0] != 'all') ? ') AND ' : '';
		// Join login/reservations on memberid
		$where .= ' (l.memberid=res.memberid) ';
		/**********************************************
		* Determine what resources to search for
		* 
		* If the first item in the array is string "all",
		* then search on all resources.
		* Else get each machid passed in and search just
		* for those resources.
		**********************************************/
		// Add "AND" to where clause
		$where .= ' AND ';
		
		if ($machids[0] != 'all') {
			// Join on specific pis
			$where .= '(rs.machid=?';
			array_push($values, $machids[0]);
		}
	
		if ( (count($machids)>1) && ($machids[0] != 'all') ) {
			for ($i = 1; $i < count($machids); $i++) {
				$where .= ' OR rs.machid=?';
				array_push($values, $machids[$i]);
			}
		}
		
		// Add "AND" if WHERE clause is not empty
		$where .= ($machids[0] != 'all') ? ') AND ' : '';
		// Join resources/reservations on machid
		$where .= '(rs.machid=res.machid)';
		// Put query together
		$query .= $where . $order;

		// Prepare query
		$q = $this->db->prepare($query);
		// Execute query
		$result = $this->db->execute($q, $values);
	
		// Check query
		$this->check_for_error($result);
		
		$return = array();
		while ($rs = $result->fetchRow())
			$return[] = $this->cleanRow($rs);
		
		return $return;
	}
	
	/**
	* Return the total reservation times for each resource
	* @param array $machids machids to return
	* @return mixed resource array of machid => total time
	*/
	function get_resource_times($machids) {
		$return = array();
		
		$in = ($machids[0] != 'all') ? ' WHERE machid IN ("' . join('","', $machids) . '")' : '';
		$query = 'SELECT sum(endTime-startTime) as sum, machid FROM ' . $this->get_table('reservations') . $in . ' GROUP BY machid';
		$result = $this->db->query($query);
		
		$this->check_for_error($result);
		
		while ($rs = $result->fetchRow())
			$return[$rs['machid']] = $rs['sum'];
		
		return $return;
	}
}
?>